/* --------------------------------------------------------------------
   Module:	support.cpp
   Subject: net6.9 preprocessor support functions
   Author:	golden spud
   --------------------------------------------------------------------
   Description: stripansi(), message functions, and the like
   --------------------------------------------------------------------
   History:    (insert new entries at top of list)
   YYmmmDD/nam description
   -------------------------------------------------------------------- */

/* --- Includes ------------------------------------------------------- */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "support.h"
#include "compare.h"

/* --- Defines -------------------------------------------------------- */

/* --- Constants ------------------------------------------------------ */

/* --- Types ---------------------------------------------------------- */

/* --- Prototypes ----------------------------------------------------- */

/* --- Global Variables ----------------------------------------------- */

/* --- Local Variables ------------------------------------------------ */

/* --- Implementation ------------------------------------------------- */

// changes string being sent, does not return a pointer like deansi
void stripansi(char *str)
{
	int i = 0;
	char *pc;

	pc = str;

	while (*pc) {
		if (*pc == CTRL_A || *pc == CTRL_B) {
			if (*(pc + 1)) {
				pc += 2;

				if (toupper(*(pc - 1)) == 'X') {
					while (*pc > CTRL_B && !isspace(*pc) &&
							toupper(*pc) != 'X' && *pc != '\r' && *pc != '\n')
					{
						pc++;
					} // while

					if (toupper(*pc) == 'X') {
						pc++;
					} // if
				} // if
			} else {
				pc++;
			} // if
		} else {
			str[i] = *pc;
			i++;
			pc++;
		} // if
	} // while

	str[i] = 0;
} // stripansi()


//-----------------------------------------------------------------------
//	u_match() - Unix wildcarding
//-----------------------------------------------------------------------
int u_match(register char *s, register char *p)
{
#define CMASK	0377
#undef	QUOTE
#define QUOTE	0200
#define QMASK	(CMASK&~QUOTE)
#define NOT '!' /* might use ^ */

	register int sc, pc;

	if (s == NULL || p == NULL)
		return(0);
	while ((pc = *p++ & CMASK) != '\0') {
		sc = *s++ & QMASK;
		switch (pc) {
		case '[':
			if ((p = cclass(p, sc)) == NULL)
				return(0);
			break;

		case '?':
			if (sc == 0)
				return(0);
			break;

		case '*':
			s--;
			do {
				if (*p == '\0' || u_match(s, p))
					return(1);
			} while (*s++ != '\0');
			return(0);

		default:
			if (tolower(sc) != (tolower(pc&~QUOTE)))
				return(0);
		}
	}
	return(*s == 0);
}

//-----------------------------------------------------------------------
//	cclass() - used with u_match()
//-----------------------------------------------------------------------
static char *cclass(register char *p, register int sub)
{
	register int c, d, not, found;

	sub = tolower(sub);

	if ((not = *p == NOT) != 0)
		p++;
	found = not;
	do {
		if (*p == '\0')
			return(NULL);
		c = tolower(*p & CMASK);
		if (p[1] == '-' && p[2] != ']') {
			d = tolower(p[2] & CMASK);
			p++;
		} else
			d = c;
		if (c == sub || c <= sub && sub <= d)
			found = !not;
	} while (*++p != ']');
	return(found? p+1: NULL);
}


void getstr(FILE *f, char *str, int mlen)
{
	int l = 0;
	unsigned char ch = 1;

	while (!feof(f) && ch) {
		ch = (unsigned char)(fgetc(f));
		if ((ch != 0xff && ch != '\r') && l < mlen) {
			str[l] = ch;
			l++;
		} // if
	} // while
	str[l] = '\0';
} // getstr


void putstr(FILE *f, const char *str)
{
	const char *s2 = str;

	do {
		fputc(*s2, f);
	} while (*(s2++));
} // putstr



int get_message(FILE *f, message_t& msg)
{
	char *tempfield = (char *)calloc(1, FIELDLEN+1);
    if (tempfield == NULL) {
        fprintf(stderr, "sig69: not enough memory in 'get_message()'\n");
        exit(1);
    }

	// get and test for little 0xff 0x00 thing
	int tmpch;
	tmpch = fgetc(f);
	switch (tmpch) {
		case 0xff:
			fgetc(f);
			break;

		case 0x00:
			free(tempfield);
			return GETMSG_ENDROOM;

		case EOF:
			free(tempfield);
			return GETMSG_EOF;
	} // switch

	getstr(f, msg.number, ROOMLEN);
	if (feof(f)) {
		free(tempfield);
		return GETMSG_EOF;
	} // if

	msg.first = NULL;

	message_field_t *current = NULL;
	while (1) {
		int field_char;
		message_field_t *temp;
		temp = (message_field_t *)calloc(1, sizeof (message_field_t));

		field_char = fgetc(f);
		if (field_char == 0) {
			free(tempfield);
			return GETMSG_BADMSG;
		} // if
		if (field_char == EOF) {
			free(tempfield);
			return GETMSG_BADMSG;
		} // if

		temp->fieldid = field_char;
		getstr(f, tempfield, FIELDLEN);
		temp->fieldtext = (char *)calloc(1, strlen(tempfield)+1);
        if (temp->fieldtext == NULL) {
            fprintf(stderr, "sig69: not enough memory in 'get_message()'\n");
            exit(1);
        }
		strcpy(temp->fieldtext, tempfield);

		temp->next = NULL;
		if (current != NULL)
			current->next = temp;
		current = temp;
		if (msg.first == NULL)
			msg.first = current;

		if (field_char == 'M') {
			free(tempfield);
			return GETMSG_OK;
		} // if
	} // while
} // get_message



void put_message(FILE *f, const message_t& msg)
{
	if (msg.first == NULL)
		return;

	// put the 0xff string for start of message
	fputc(0xff, f);
	fputc(0, f);

	putstr(f, msg.number);
	message_field_t *current = msg.first;

	while (current != NULL) {
		fputc(current->fieldid, f);
		putstr(f, current->fieldtext);
		current = current->next;
	} // while
} // put_message




void kill_message(message_t& msg)
{
	while (msg.first != NULL) {
		message_field_t *temp = msg.first->next;
		free((msg.first)->fieldtext);
		free(msg.first);
		msg.first = temp;
	} // while

	msg.first = NULL;
	msg.number[0] = 0;

} // kill_message



int search_message(const message_t& msg, char field, char *test, int dansi)
{
	message_field_t *current = msg.first;

	while (current != NULL) {
		if (current->fieldid == field) {
			if (dansi)
                stripansi(current->fieldtext);
            if (stricmp(current->fieldtext, test) == STREQUAL)
				return 1;
		} // if
		current = current->next;
	} // while
	return 0;
} // search_message




/* --- End of File ---------------------------------------------------- */
