/* --------------------------------------------------------------------
   Module:	vpath.cpp
   Subject: Verbose Path main module
   Author:	golden spud
   --------------------------------------------------------------------
   Description: adds a verbose path to public net6.9 messages for
				purposes of checking how long it takes messages to get
				from one point to another.
   --------------------------------------------------------------------
   History:    (insert new entries at top of list)
   YYmmmDD/nam description
   -------------------------------------------------------------------- */

/* --- Includes ------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "support.h"

/* --- Defines -------------------------------------------------------- */

/* --- Constants ------------------------------------------------------ */

/* --- Types ---------------------------------------------------------- */

/* --- Prototypes ----------------------------------------------------- */

/* --- Global Variables ----------------------------------------------- */

/* --- Local Variables ------------------------------------------------ */

/* --- Implementation ------------------------------------------------- */

int main(int argc, char **argv)
{
    if (argc != 2) {
        fprintf(stderr, "%s address\n", argv[0]);
        return 1;
    }

    FILE *fi = fopen("msg.69", "rb");
    if (!fi) {
        return 0;
    }

    FILE *fo = fopen("$!$!$!$!.$!$", "wb");
    if (!fo) {
        return 0;
    }

    char miscname[ROOMLEN+1];

    getstr(fi, miscname, ROOMLEN);      // first is address
    putstr(fo, miscname);
    printf("Packet from %s\n", miscname);

    getstr(fi, miscname, ROOMLEN);
    putstr(fo, miscname);
    printf("%s", miscname);

    int msgerr;
    message_t msg;
    msg.first = NULL;

    int done_processing_messages = 0;
    while (!done_processing_messages) {
        kill_message(msg);
        msgerr = get_message(fi, msg);

        switch (msgerr) {
            case GETMSG_EOF:
                done_processing_messages = 1;
                break;

            case GETMSG_BADMSG:
                fprintf(stderr, "\nbad message found!\n");
                return 1;

            case GETMSG_ENDROOM:
                // get the next roomname
                putstr(fo, "");
                getstr(fi, miscname, ROOMLEN);
                putstr(fo, miscname);
                printf("\n%s", miscname);
                break;

            case GETMSG_OK:
                {
                    printf(".");

                    // traverse until M is found...
                    // after it is found, oldf contains the field just
                    // before it.
                    message_field_t* f;
                    message_field_t* oldf;
                    f = oldf = msg.first;
                    while (1) {
                        if (!f) {
                            fprintf(stderr, "\nno M field found!\n");
                            return 1;
                        }
                        if (f->fieldid == 'M')
                            break;
                        oldf = f;
                        f = f->next;
                    }

                    message_field_t* newf = new message_field_t;
                    oldf->next = newf;
                    newf->next = f;
                    newf->fieldid = '%';

                    // now that we don't need f, traverse through msg to
                    // find D field.
                    f = msg.first;
                    time_t d = 0L;
                    while (f) {
                        if (f->fieldid == 'D') {
                            d = atol(f->fieldtext);
                        }
                        f = f->next;
                    }
                    if (!d) {
                        fprintf(stderr, "\nno D field found!\n");
                        return 1;
                    }
                    time_t now = time(NULL);

                    int negative;
                    time_t dif;
                    if (now < d) {
                        negative = 1;
                        dif = 0;
                    } else {
                        negative = 0;
                        dif = now - d;
                    }

                    time_t days =       (dif / (60L*60L*24L));
                    time_t hours =     ((dif % (60L*60L*24L)) / (60L*60L));
                    time_t minutes =  (((dif % (60L*60L*24L)) % (60L*60L)) / 60L);
                    time_t seconds = ((((dif % (60L*60L*24L)) % (60L*60L)) % 60L) / 1L);

                    char *difstr = new char[81];
                    if (!difstr) {
                        fprintf(stderr, "\nerror allocating difstr\n");
                        return 1;
                    }
                    char wow[81];
                    strcpy(wow, ctime(&now));
                    if (wow[strlen(wow)-1] == '\n')
                        wow[strlen(wow)-1] = 0;
                    sprintf(
                        difstr,
                        "%%%s: %lu, %lu:%02lu:%02lu", argv[1], now, days, hours,
                        minutes);

//                        "rcvd @ %s: %s, %02lu:%02lu:%02lu:%02lu after msg date",
 //                       argv[1],
  //                      wow,
   //                     days,
    //                    hours,
     //                   minutes,
      //                  seconds
       //             );
                    newf->fieldtext = difstr;
//                    printf("\n%s (%lu-%lu=%lu)", difstr, d, now, dif);

                    put_message(fo, msg);
                    kill_message(msg);
                }
                break;
        }
    }

    fclose(fi);
    fclose(fo);

    unlink("msg.69");
    rename("$!$!$!$!.$!$", "msg.69");

    return 0;
}

/* --- End of File ---------------------------------------------------- */
